import { SWR, SWRKey, SWRMutateValue, SWROptions, SWRMutateOptions, SWRRevalidateOptions, CacheClearOptions } from 'swrev';
import { Ref, ComputedRef, WatchStopHandle } from 'vue';
export interface SWRResponse<D = any, E = Error> {
    data: Ref<D | undefined>;
    error: Ref<E | undefined>;
    mutate: (value: D, ops?: Partial<SWRMutateOptions<D>> | undefined) => void;
    revalidate: (ops?: Partial<SWRRevalidateOptions<D>> | undefined) => void;
    clear: (ops?: Partial<CacheClearOptions> | undefined) => void;
    unsubscribe: WatchStopHandle;
    isLoading: ComputedRef<boolean>;
    isValid: ComputedRef<boolean>;
    promise: Promise<Ref<D>>;
}
export interface SWRSuspenseResponse<D = any> {
    data: Ref<D>;
    mutate: (value: D, ops?: Partial<SWRMutateOptions<D>> | undefined) => void;
    revalidate: (ops?: Partial<SWRRevalidateOptions<D>> | undefined) => void;
    clear: (ops?: Partial<CacheClearOptions> | undefined) => void;
    unsubscribe: WatchStopHandle;
}
export declare type SuspenseSWRBuilder = <D = any>(key: SWRKey, options?: Partial<SWROptions<D>>) => Promise<SWRSuspenseResponse<D>>;
export declare class VSWR extends SWR {
    /**
     * Use the SWR with a vue application.
     * Queries the data.
     */
    query<D = any, E = Error>(key?: SWRKey | (() => SWRKey | undefined), options?: Partial<SWROptions<D>>): SWRResponse<D, E>;
    /**
     * Query with suspense support.
     */
    querySuspense<D = any>(key?: SWRKey | (() => SWRKey | undefined), options?: Partial<SWROptions<D>>): Promise<SWRSuspenseResponse<D>>;
}
/**
 * Creates a mew SWR instance and exports basic methods to
 * work with without the need for method calling.
 */
export declare const createSWR: <D = any>(options?: Partial<SWROptions<D>> | undefined) => VSWR;
/**
 * Default SWR instance. Can be replaced
 * with the `createDefaultSWR` function.
 */
export declare let swr: VSWR;
/**
 * Creates and sets a default SWR instance given
 * the options.
 */
export declare const createDefaultSWR: <D = any>(options?: Partial<SWROptions<D>> | undefined) => VSWR;
/**
 * Gets the data of the given key. Keep in mind
 * this data will be stale and revalidate in the background
 * unless specified otherwise.
 */
export declare const subscribeData: <D>(key: SWRKey | undefined, onData: (value: D) => any) => () => void;
/**
 * Subscribes to errors on the given key.
 */
export declare const subscribeErrors: <E>(key: SWRKey | undefined, onError: (error: E) => any) => () => void;
/**
 * Gets the current cached data of the given key.
 * This does not trigger any revalidation nor mutation
 * of the data.
 * - If the data has never been validated
 * (there is no cache) it will return undefined.
 * - If the item is pending to resolve (there is a request
 * pending to resolve) it will return undefined.
 */
export declare const get: <D = any>(key?: string | undefined) => D | undefined;
/**
 * Gets an element from the cache. The difference
 * with the get is that this method returns a promise
 * that will resolve the the value. If there's no item
 * in the cache, it will wait for it before resolving.
 */
export declare const getWait: <D = any>(key: SWRKey) => Promise<D>;
/**
 * Use a SWR value given the key and
 * subscribe to future changes.
 */
export declare const subscribe: <D = any, E = Error>(key: SWRKey | undefined | (() => SWRKey | undefined), onData: (value: D) => void, onError: (error: E) => void, options?: Partial<SWROptions<D>> | undefined) => {
    unsubscribe: () => void;
};
/**
 * Use the SWR with a vue application.
 */
export declare const query: <D = any, E = Error>(key?: string | (() => SWRKey | undefined) | undefined, options?: Partial<SWROptions<D>> | undefined) => SWRResponse<D, E>;
/**
 * Query with suspense support.
 */
export declare const querySuspense: <D = any>(key?: string | (() => SWRKey | undefined) | undefined, options?: Partial<SWROptions<D>> | undefined) => Promise<SWRSuspenseResponse<D>>;
/**
 * Mutates the data of a given key with a new value.
 * This is used to replace the cache contents of the
 * given key manually.
 */
export declare const mutate: <D = any>(key?: string | undefined, value?: SWRMutateValue<D> | undefined, options?: Partial<SWRMutateOptions<any>> | undefined) => void;
/**
 * Revalidates the key and mutates the cache if needed.
 */
export declare const revalidate: <D>(key?: string | undefined, options?: Partial<SWRRevalidateOptions<D>> | undefined) => void;
/**
 * Clear the specified keys from the cache. If no keys
 * are specified, it clears all the cache keys.
 */
export declare const clear: (keys?: string | string[] | null | undefined, options?: Partial<CacheClearOptions> | undefined) => void;
